'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

exports.__esModule = true;

var _MapGenerator = require('./map-generator');

var _MapGenerator2 = _interopRequireWildcard(_MapGenerator);

var _warnOnce = require('./warn-once');

var _warnOnce2 = _interopRequireWildcard(_warnOnce);

var _Result = require('./result');

var _Result2 = _interopRequireWildcard(_Result);

var _parse = require('./parse');

var _parse2 = _interopRequireWildcard(_parse);

var _Root = require('./root');

var _Root2 = _interopRequireWildcard(_Root);

var Promise = global.Promise || require('es6-promise').Promise;

function isPromise(obj) {
    return typeof obj === 'object' && typeof obj.then === 'function';
}

var LazyResult = (function () {
    function LazyResult(processor, css, opts) {
        _classCallCheck(this, LazyResult);

        this.stringified = false;
        this.processed = false;

        var root = undefined;
        if (css instanceof _Root2['default']) {
            root = css;
        } else if (css instanceof LazyResult || css instanceof _Result2['default']) {
            root = css.root;
            if (css.map && typeof opts.map === 'undefined') {
                opts.map = { prev: css.map };
            }
        } else {
            root = _parse2['default'](css, opts);
        }

        this.result = new _Result2['default'](processor, root, opts);
    }

    LazyResult.prototype.warnings = function warnings() {
        return this.sync().warnings();
    };

    LazyResult.prototype.toString = function toString() {
        return this.css;
    };

    LazyResult.prototype.then = function then(onFulfilled, onRejected) {
        return this.async().then(onFulfilled, onRejected);
    };

    LazyResult.prototype['catch'] = function _catch(onRejected) {
        return this.async()['catch'](onRejected);
    };

    LazyResult.prototype.asyncTick = function asyncTick(plugins, resolve, reject) {
        var _this = this;

        if (plugins.length === 0) {
            return resolve();
        }var promise = this.run(plugins.shift());
        if (isPromise(promise)) {
            promise.then(function () {
                _this.asyncTick(plugins, resolve, reject);
            })['catch'](function (error) {
                reject(error);
            });
        } else {
            this.asyncTick(plugins, resolve, reject);
        }
    };

    LazyResult.prototype.async = function async() {
        var _this2 = this;

        if (this.processed) {
            return Promise.resolve().then(function () {
                return _this2.stringify();
            });
        }
        if (this.processing) {
            return this.processing;
        }

        var plugins = this.processor.plugins.slice(0);
        this.processing = new Promise(function (resolve, reject) {
            _this2.asyncTick(plugins, resolve, reject);
        }).then(function () {
            _this2.processed = true;
            return _this2.stringify();
        });

        return this.processing;
    };

    LazyResult.prototype.sync = function sync() {
        if (this.processed) {
            return this.result;
        }this.processed = true;

        if (this.processing) {
            throw 'Use process(css).then(cb) to work with async plugins';
        }

        for (var _iterator = this.result.processor.plugins, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
            var _ref;

            if (_isArray) {
                if (_i >= _iterator.length) break;
                _ref = _iterator[_i++];
            } else {
                _i = _iterator.next();
                if (_i.done) break;
                _ref = _i.value;
            }

            var plugin = _ref;

            var promise = this.run(plugin);
            if (isPromise(promise)) {
                throw 'Use process(css).then(cb) to work with async plugins';
            }
        }

        return this.result;
    };

    LazyResult.prototype.run = function run(plugin) {
        this.result.lastPlugin = plugin;

        var returned = undefined;
        try {
            returned = plugin(this.result.root, this.result);
        } catch (error) {
            if (plugin.postcssVersion) {
                var pluginName = plugin.postcssPlugin;
                var pluginVersion = plugin.postcssVersion;
                var runtimeVersion = this.result.processor.version;
                var a = pluginVersion.split('.');
                var b = runtimeVersion.split('.');

                if (a[0] !== b[0] || parseInt(a[1]) > parseInt(b[1])) {
                    _warnOnce2['default']('' + pluginName + ' is based on ' + ('PostCSS ' + pluginVersion + ' but you use it ') + ('with PostCSS ' + runtimeVersion + '. ') + 'Maybe this is a source of error below.');
                }
            }
            throw error;
        }

        if (returned instanceof _Root2['default']) {
            this.result.root = returned;
        } else {
            return returned;
        }
    };

    LazyResult.prototype.stringify = function stringify() {
        if (this.stringified) {
            return this.result;
        }this.stringified = true;

        this.sync();
        var map = new _MapGenerator2['default'](this.result.root, this.result.opts);

        var _map$generate = map.generate();

        this.result.css = _map$generate[0];
        this.result.map = _map$generate[1];

        return this.result;
    };

    _createClass(LazyResult, [{
        key: 'processor',
        get: function () {
            return this.result.processor;
        }
    }, {
        key: 'opts',
        get: function () {
            return this.result.opts;
        }
    }, {
        key: 'css',
        get: function () {
            return this.stringify().css;
        }
    }, {
        key: 'map',
        get: function () {
            return this.stringify().map;
        }
    }, {
        key: 'root',
        get: function () {
            return this.sync().root;
        }
    }, {
        key: 'messages',
        get: function () {
            return this.sync().messages;
        }
    }]);

    return LazyResult;
})();

exports['default'] = LazyResult;
module.exports = exports['default'];